#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2024 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

#
# From 7.2.2, docker enterprise support is added. Before to 7.2.2, when system connector gets installed,
# it used to create the symlink in /opt/cyops-integrations/integrations/connectors. Since from specfile
# of system connectors, manage.py is run as 'sudo -u nginx', symlink owner is set to 'nginx'. Since nginx
# was creating symlink in /opt/cyops-integrations/integrations/connectors, ACL permission was required to this
# dir. Because of the ACL, it was causing issue on docker having backing filesystem as 'xfs', while restoring
# these symlinks. Hence create soft links using 'fsr-integrations' user and remove the ACL from
# /opt/cyops-integrations/integrations/connectors for 'nginx'. User 'nginx' don't need ACL on mentioned dir,
# as connector installation, uninstallation is done at dir /opt/cyops/configs/integrations/connectors from 7.2.2.
# When user is 'nginx', it will be creating symlink as 'sudo -u fsr-integrations' from integrations-sdk.
#
configure_sudoers_nginx() {
    local new_user=$1
    local user_nginx='nginx'
    local SUDOERS="/etc/sudoers"
    local perms_sudoers="$user_nginx ALL=($new_user) NOPASSWD: /bin/ln -fs /opt/cyops/configs/integrations/connectors/* /opt/cyops-integrations/integrations/connectors/*, /bin/kill, /usr/bin/kill"
    echo "Adding sudoers for user [$user_nginx]"
    if grep -qE "^\s*$user_nginx\s+ALL\s*=\s*\(\s*$new_user\s*\)\s+NOPASSWD\s*:\s*/bin/ln" $SUDOERS; then
        # replace old line
        perms_sudoers=$(echo "$perms_sudoers" | sed  -e 's:/:\\\/:g')
        sed -i "s/^\s*$user_nginx \+ALL *= *( *$new_user *) *NOPASSWD *:.*\/bin\/ln.*$/$perms_sudoers/g" $SUDOERS
        return
    fi
    echo "$perms_sudoers" >> $SUDOERS
}

configure_sudoers_integrations() {
    local new_user=$1
    local SUDOERS="/etc/sudoers"
    local perms_sudoers="$new_user ALL=(ALL) NOPASSWD: /bin/csadm package *, /bin/hamanager"
    echo "Adding sudoers for user [$new_user]"
    if grep -qE "^\s*$new_user\s+ALL\s*=\s*\(\s*ALL\s*\)\s+NOPASSWD\s*:" $SUDOERS; then
        # replace old line
        perms_sudoers=$(echo "$perms_sudoers" | sed  -e 's:/:\\\/:g')
        sed -i "s/^\s*$new_user \+ALL *= *( *ALL *) *NOPASSWD *:.*$/$perms_sudoers/g" $SUDOERS
        return
    fi
    echo "$perms_sudoers" >> $SUDOERS
}

configure_user_integrations() {
    local new_user=$1
    local old_user=$2
    local nginx_user="nginx"
    local perm_dirs="/var/log/cyops/cyops-integrations/ \
                     /opt/cyops-integrations/.env/ \
                     /opt/cyops/configs/integrations/" 
    for perm_dir in $perm_dirs; do
        setfacl -Rm u:${new_user}:rwx,u:${nginx_user}:rwx $perm_dir
        setfacl -Rdm u:${new_user}:rwx,u:${nginx_user}:rwx $perm_dir
    done
    configure_sudoers_integrations "$new_user"
    #
    # From 7.2.2, changing the socket for integrations at /var/run/uwsgi-integrations/ from /var/run/uwsgi/.
    # This is required for docker on Mac. For docker enterprise, we start the container as --tmpfs
    # /run i.e /run is RAM. Changing the ACL on RAM is not working on Mac, and hence we need separate
    # directory for integrations socket. Note that this is not failing for enterprise OVA or docker
    # on CentOS or RHEL, or on FMG/FAZ. It is failing only for Mac. But since changing the socket
    # location is not error prone, we are changing the socket location for integrations.
    # Below call is not required for fresh install, but required for upgrade from version below
    # 7.2.2. Since this script gets called from specfile for both fresh install and upgrade, adding
    # call here.
    #
    local intg_ngx_cnf="/etc/nginx/conf.d/cyops-integrations.conf"
    if ! grep -q "unix:/var/run/uwsgi-integrations/" $intg_ngx_cnf; then
        sed -i "s#unix:/var/run/uwsgi/#unix:/var/run/uwsgi-integrations/#g" $intg_ngx_cnf
        systemctl restart nginx
    fi
}

move_connectors_development() {
    local old_dir="/opt/cyops-integrations/integrations/connector_development/"
    local new_dir="/opt/cyops/configs/integrations/connectors_development/"
    local old_dir_dev_connectors=$(find $old_dir -maxdepth 1 -name '*_dev' -type d)
    for old_dir_dev_connector in $old_dir_dev_connectors; do
        mv $old_dir_dev_connector $new_dir
    done
}

############
# Main/main/MAIN starts here
#########
echo "Stopping uWSGI"
systemctl stop uwsgi
new_user="fsr-integrations"
old_user="nginx"
configure_user_integrations "$new_user" "$old_user"
configure_sudoers_nginx "$new_user"
#
# move_connectors_development can be removed once 7.2.2 becomes base release for upgrade.
# Even if we don't remove, no issues, as code is idempotent.
#
move_connectors_development

echo "Starting uWSGI"
systemctl start uwsgi
exit 0
